@@ -27,25 +27,19 @@
  * for helping me make this possible
  */
 
-#include <stdlib.h>
-#include <string.h>
-#include <stdio.h>
 #include <math.h>
 
-#include <X11/Xatom.h>
-#include <X11/extensions/Xrender.h>
-
-#include <compiz.h>
+#include <compiz-core.h>
 #include "snow_options.h"
 
 #define GET_SNOW_DISPLAY(d)                            \
-    ((SnowDisplay *) (d)->privates[displayPrivateIndex].ptr)
+    ((SnowDisplay *) (d)->base.privates[displayPrivateIndex].ptr)
 
 #define SNOW_DISPLAY(d)                                \
     SnowDisplay *sd = GET_SNOW_DISPLAY (d)
 
 #define GET_SNOW_SCREEN(s, sd)                         \
-    ((SnowScreen *) (s)->privates[(sd)->screenPrivateIndex].ptr)
+    ((SnowScreen *) (s)->base.privates[(sd)->screenPrivateIndex].ptr)
 
 #define SNOW_SCREEN(s)                                 \
     SnowScreen *ss = GET_SNOW_SCREEN (s, GET_SNOW_DISPLAY (s->display))
@@ -131,9 +125,9 @@ static void
 snowMove (CompDisplay *d,
 	  SnowFlake   *sf)
 {
-    float tmp = 1.0f / (100.0f - snowGetSnowSpeed (d));
+    float tmp = 1.0f / (101.0f - snowGetSnowSpeed (d));
     int   snowUpdateDelay = snowGetSnowUpdateDelay (d);
-
+    
     sf->x += (sf->xs * (float) snowUpdateDelay) * tmp;
     sf->y += (sf->ys * (float) snowUpdateDelay) * tmp;
     sf->z += (sf->zs * (float) snowUpdateDelay) * tmp;
@@ -513,8 +507,10 @@ snowInitScreen (CompPlugin *p,
     SNOW_DISPLAY (s->display);
 
     ss = calloc (1, sizeof(SnowScreen));
+    if (!ss)
+	return FALSE;
 
-    s->privates[sd->screenPrivateIndex].ptr = ss;
+    s->base.privates[sd->screenPrivateIndex].ptr = ss;
 
     ss->s = s;
     ss->snowTexturesLoaded = 0;
@@ -523,6 +519,11 @@ snowInitScreen (CompPlugin *p,
     ss->displayListNeedsUpdate = FALSE;
 
     ss->allSnowFlakes = snowFlake = malloc (numFlakes * sizeof (SnowFlake));
+    if (!snowFlake)
+    {
+	free (ss);
+	return FALSE;
+    }
 
     for (i = 0; i < numFlakes; i++)
     {
@@ -657,8 +658,13 @@ snowInitDisplay (CompPlugin  *p,
 {
     CompOption  *texOpt;
     SnowDisplay *sd;
-    
+
+    if (!checkPluginABI ("core", CORE_ABIVERSION))
+	return FALSE;
+
     sd = malloc (sizeof (SnowDisplay));
+    if (!sd)
+	return FALSE;
 
     sd->screenPrivateIndex = allocateScreenPrivateIndex (d);
     if (sd->screenPrivateIndex < 0)
@@ -667,7 +673,7 @@ snowInitDisplay (CompPlugin  *p,
 	return FALSE;
     }
 	
-    snowSetToggleInitiate (d, snowToggle);
+    snowSetToggleKeyInitiate (d, snowToggle);
     snowSetNumSnowflakesNotify (d, snowDisplayOptionChanged);
     snowSetSnowSizeNotify (d, snowDisplayOptionChanged);
     snowSetSnowUpdateDelayNotify (d, snowDisplayOptionChanged);
@@ -677,7 +683,7 @@ snowInitDisplay (CompPlugin  *p,
     sd->snowTexFiles = texOpt->value.list.value;
     sd->snowTexNFiles = texOpt->value.list.nValue;
 
-    d->privates[displayPrivateIndex].ptr = sd;
+    d->base.privates[displayPrivateIndex].ptr = sd;
 
     return TRUE;
 }
@@ -692,6 +698,32 @@ snowFiniDisplay (CompPlugin  *p,
     free (sd);
 }
 
+static CompBool
+snowInitObject (CompPlugin *p,
+		CompObject *o)
+{
+    static InitPluginObjectProc dispTab[] = {
+	(InitPluginObjectProc) 0, /* InitCore */
+	(InitPluginObjectProc) snowInitDisplay,
+	(InitPluginObjectProc) snowInitScreen
+    };
+
+    RETURN_DISPATCH (o, dispTab, ARRAY_SIZE (dispTab), TRUE, (p, o));
+}
+
+static void
+snowFiniObject (CompPlugin *p,
+		CompObject *o)
+{
+    static FiniPluginObjectProc dispTab[] = {
+	(FiniPluginObjectProc) 0, /* FiniCore */
+	(FiniPluginObjectProc) snowFiniDisplay,
+	(FiniPluginObjectProc) snowFiniScreen
+    };
+
+    DISPATCH (o, dispTab, ARRAY_SIZE (dispTab), (p, o));
+}
+
 static Bool
 snowInit (CompPlugin *p)
 {
@@ -708,27 +740,13 @@ snowFini (CompPlugin *p)
     freeDisplayPrivateIndex (displayPrivateIndex);
 }
 
-static int
-snowGetVersion (CompPlugin *p,
-		int        version)
-{
-    return ABIVERSION;
-}
-
 CompPluginVTable snowVTable = {
     "snow",
-    snowGetVersion,
     0,
     snowInit,
     snowFini,
-    snowInitDisplay,
-    snowFiniDisplay,
-    snowInitScreen,
-    snowFiniScreen,
-    0,
-    0,
-    0,
-    0,
+    snowInitObject,
+    snowFiniObject,
     0,
     0
 };
